@@ -1,9 +1,11 @@
-from pathlib import Path
-from datetime import datetime, timedelta
-import pandas as pd
-import numpy as np
 import logging
-from seabirdfilehandler import DataFile, Parameters, CnvProcessingSteps
+from datetime import datetime, timedelta
+from pathlib import Path
+
+import numpy as np
+import pandas as pd
+
+from seabirdfilehandler import CnvProcessingSteps, DataFile, Parameters
 
 logger = logging.getLogger(__name__)
 
@@ -154,20 +156,20 @@ class CnvFile(DataFile):
             cnv_like_row = "".join(
                 (lambda column: f"{str(column):>11}")(value) for value in row
             )
-            cnv_out.append(cnv_like_row + "\n")
+            cnv_out.append(cnv_like_row + "\r\n")
         return cnv_out
 
     def array2cnv(self) -> list:
         result = []
         for row in self.parameters.full_data_array:
             formatted_row = "".join(str(elem).rjust(11) for elem in row)
-            result.append(formatted_row + "\n")
+            result.append(formatted_row + "\r\n")
         return result
 
     def to_cnv(
         self,
         file_name: Path | str | None = None,
-        use_dataframe: bool = True,
+        use_dataframe: bool = False,
     ):
         """
         Writes the values inside of this instance as a new cnv file to disc.
@@ -207,12 +209,15 @@ class CnvFile(DataFile):
         self.processing_info = self.processing_steps._form_processing_info()
         self.header = [
             *[f"* {data}" for data in self.sbe9_data[:-1]],
-            *[f"** {data}" for data in self.metadata_list],
+            *[
+                f"** {key} = {value}\r\n" if value else f"** {key}\r\n"
+                for key, value in self.metadata.items()
+            ],
             f"* {self.sbe9_data[-1]}",
             *[f"# {data}" for data in self.data_table_description],
             *[f"# {data}" for data in self.sensor_data],
             *[f"# {data}" for data in self.processing_info],
-            "*END*\n",
+            "*END*\r\n",
         ]
         self.data = self.array2cnv()
         self.file_data = [*self.header, *self.data]
@@ -241,12 +246,16 @@ class CnvFile(DataFile):
 
         """
         if "Station" in self.metadata:
-            self.parameters.create_parameter(
-                data=self.metadata["Station"],
-                name="Event",
-            )
-            return True
-        return False
+            data_value = self.metadata["Station"]
+            return_value = True
+        else:
+            data_value = None
+            return_value = False
+        self.parameters.create_parameter(
+            data=data_value,
+            name="Event",
+        )
+        return return_value
 
     def add_position_columns(self) -> bool:
         """
