@@ -1,10 +1,12 @@
 from __future__ import annotations
-from typing import Tuple
+
+import logging
 import re
 from collections import UserDict
+from typing import Tuple
+
 import numpy as np
 import pandas as pd
-import logging
 
 logger = logging.getLogger(__name__)
 
@@ -34,18 +36,28 @@ class Parameters(UserDict):
         metadata: list,
         only_header: bool = False,
     ):
-        self.raw_input_data = data
-        self.raw_metadata = metadata
-        self.differentiate_table_description()
-        self.metadata, self.duplicate_columns = self.reading_data_header(
+        self.data = {}
+        self.differentiate_table_description(metadata)
+        parsed_metadata, self.duplicate_columns = self.reading_data_header(
             metadata
         )
         if not only_header:
-            self.full_data_array = self.create_full_ndarray()
-            self.data = self.create_parameter_instances()
+            self.full_data_array = self.create_full_ndarray(data)
+            self.create_parameter_instances(
+                self.full_data_array, parsed_metadata
+            )
 
-    def get_parameter_names(self) -> list[str]:
-        return [parameter["name"] for parameter in self.metadata.values()]
+    def get_full_data_array(self) -> np.ndarray:
+        return np.array([parameter.data for parameter in self.data.values()]).T
+
+    def get_names(self) -> list[str]:
+        return [parameter.name for parameter in self.data.values()]
+
+    def get_metadata(self) -> dict[str, dict]:
+        return {
+            parameter.name: parameter.metadata
+            for parameter in self.data.values()
+        }
 
     def get_parameter_list(self) -> list[Parameter]:
         """ """
@@ -66,9 +78,6 @@ class Parameters(UserDict):
         A numpy array of the same shape as the cnv files data table
 
         """
-        data_table = (
-            self.raw_input_data if len(data_table) == 0 else data_table
-        )
         n = 11
         row_list = []
         for line in data_table:
@@ -80,9 +89,63 @@ class Parameters(UserDict):
             )
         return np.array(row_list, dtype=float)
 
+    def sort_parameters(
+        self,
+        top: list = [
+            "depSM",
+            "prDM",
+            "t090C",
+            "t190C",
+            "sal00",
+            "sal11",
+            "sbox0Mm/Kg",
+            "sbox1Mm/Kg",
+            "flECO-AFL",
+            "turbWETntu0",
+            "par",
+            "spar",
+        ],
+        bottom: list = [
+            "gsw_densityA0",
+            "gsw_densityA1",
+            "gsw_saA0",
+            "gsw_saA1",
+            "gsw_ctA0",
+            "gsw_ctA1",
+            "sbeox0ML/L",
+            "sbeox1ML/L",
+            "c0mS/cm",
+            "c1mS/cm",
+            "latitude",
+            "longitude",
+            "flag",
+        ],
+    ) -> dict:
+        # ensure parameters at the top
+        new_data = {}
+        for shortname in top:
+            for param in self.data.values():
+                if shortname == param.name:
+                    new_data[shortname] = param
+
+        # ensure parameters at the bottom
+        bottom_data = {}
+        for shortname in bottom:
+            for param in self.data.values():
+                if shortname == param.name:
+                    bottom_data[shortname] = param
+
+        for param in self.data.values():
+            if param.name not in [*top, *bottom]:
+                new_data[param.name] = param
+
+        self.data = {**new_data, **bottom_data}
+        return new_data
+
     def create_parameter_instances(
         self,
-        metadata: dict[str, dict] = {},
+        array_data: np.ndarray,
+        metadata: dict[str, dict],
     ) -> dict[str, Parameter]:
         """
         Differentiates the individual parameter columns into separate parameter
@@ -99,30 +162,24 @@ class Parameters(UserDict):
         A dictionary of parameter instances
 
         """
-        metadata = (
-            self.metadata if len(list(metadata.keys())) == 0 else metadata
-        )
         parameter_dict = {}
         list_of_metadata_shortnames = list(metadata.keys())
         # if column number and metadata number is different, we are propably
         # working with duplicate_columns and will drop the duplicates
-        if self.full_data_array.shape[1] != len(list_of_metadata_shortnames):
-            self.full_data_array = np.delete(
-                self.full_data_array, self.duplicate_columns, 1
-            )
-            assert self.full_data_array.shape[1] == len(
-                list_of_metadata_shortnames
-            )
+        if array_data.shape[1] != len(list_of_metadata_shortnames):
+            array_data = np.delete(array_data, self.duplicate_columns, 1)
+            assert array_data.shape[1] == len(list_of_metadata_shortnames)
             # rewrite the column number in the metadata header
             self.data_table_stats["nquan"] = str(
                 int(self.data_table_stats["nquan"])
                 - len(self.duplicate_columns)
             )
-        for i in range(self.full_data_array.shape[1]):
-            column_data = self.full_data_array[:, i]
+        for i in range(array_data.shape[1]):
             key = list_of_metadata_shortnames[i]
-            parameter_dict[key] = Parameter(
-                data=column_data, metadata=metadata[key]
+            parameter_dict[key] = self.create_parameter(
+                data=array_data[:, i],
+                metadata=metadata[key],
+                name=key,
             )
         return parameter_dict
 
@@ -130,17 +187,32 @@ class Parameters(UserDict):
         """Recreates the data table descriptions, like column names and spans
         from the structured dictionaries these values were stored in."""
         new_table_info = []
-        for key, value in self.data_table_stats.items():
-            new_table_info.append(f"{key} = {value}\n")
-        for index, (name, _) in enumerate(self.data_table_names_and_spans):
-            new_table_info.append(f"name {index} = {name}\n")
-        for index, (_, span) in enumerate(self.data_table_names_and_spans):
-            new_table_info.append(f"span {index} = {span}\n")
+        # 'data table stats'
+        data_array = self.get_full_data_array()
+        new_table_info.append(f"nquan = {data_array.shape[1]}\r\n")
+        new_table_info.append(f"nvalues = {data_array.shape[0]}\r\n")
+        units = (
+            self.data_table_stats["units"]
+            if "units" in list(self.data_table_stats.keys())
+            else "specified"
+        )
+        new_table_info.append(f"units = {units}\r\n")
+        # 'data tables names'
+        for index, metadata in enumerate(self.get_metadata().values()):
+            new_table_info.append(
+                f"name {index} = {metadata['shortname']}: {metadata['longinfo']}\r\n"
+            )
+        # 'data table spans'
+        for index, (minimum, maximum) in enumerate(self.get_spans()):
+            new_table_info.append(
+                f"span {index} = {minimum:2.4f}, {maximum:2.4f}\r\n"
+            )
+        # 'data table misc'
         for key, value in self.data_table_misc.items():
-            new_table_info.append(f"{key} = {value}\n")
+            new_table_info.append(f"{key} = {value}\r\n")
         return new_table_info
 
-    def differentiate_table_description(self):
+    def differentiate_table_description(self, metadata: list):
         """
         The original method that structures data table metadata.
 
@@ -151,12 +223,12 @@ class Parameters(UserDict):
         column_names = []
         column_value_spans = []
         post = []
-        for line in self.raw_metadata:
+        for line in metadata:
             if line.startswith("name"):
-                column_names.append(line.split("=")[1].strip())
+                column_names.append(line.split("=", 1)[1].strip())
             elif line.startswith("span"):
                 past_spans = True
-                column_value_spans.append(line.split("=")[1].strip())
+                column_value_spans.append(line.split("=", 1)[1].strip())
             else:
                 if not past_spans:
                     pre.append(line)
@@ -164,7 +236,7 @@ class Parameters(UserDict):
                     post.append(line)
         assert len(column_names) == len(column_value_spans)
         self.data_table_stats = {
-            line.split("=")[0].strip(): line.split("=")[1].strip()
+            line.split("=")[0].strip(): line.split("=", 1)[1].strip()
             for line in pre
         }
         self.data_table_names_and_spans = [
@@ -172,7 +244,7 @@ class Parameters(UserDict):
             for name, span in zip(column_names, column_value_spans)
         ]
         self.data_table_misc = {
-            line.split("=")[0].strip(): line.split("=")[1].strip()
+            line.split("=")[0].strip(): line.split("=", 1)[1].strip()
             for line in post
         }
 
@@ -186,34 +258,21 @@ class Parameters(UserDict):
             The new parameter
 
         """
-        position_index = -1
-        # add to parameter dict at given
+        # add to parameter dict at given position
         if position:
             new_dict = {}
-            for index, (key, value) in enumerate(self.data.items()):
+            for key, value in self.data.items():
                 new_dict[key] = value
                 if key == position:
                     new_dict[parameter.name] = parameter
-                    position_index = index + 1
             self.data = new_dict
 
         else:
             self.data[parameter.name] = parameter
 
-        # update metadata dict
-        self.metadata = {
-            parameter.name: parameter.metadata
-            for parameter in self.data.values()
-        }
-        # add to the data array if data
-        if parameter.type == "data":
-            self.full_data_array = np.insert(
-                self.full_data_array, position_index, parameter.data, axis=1
-            )
-
     def create_parameter(
         self,
-        data: np.ndarray | int | float | str,
+        data: np.ndarray | int | float | str | None,
         metadata: dict = {},
         name: str = "",
         position: str = "",
@@ -254,7 +313,7 @@ class Parameters(UserDict):
         if not isinstance(data, np.ndarray):
             data = np.full(
                 fill_value=data,
-                shape=self.full_data_array.shape[0],
+                shape=self.get_full_data_array().shape[0],
             )
         parameter = Parameter(data=data, metadata=metadata)
         self.add_parameter(parameter, position)
@@ -355,17 +414,17 @@ class Parameters(UserDict):
 
 
         """
-        if len(header_info) == 0:
-            header_info = self.raw_metadata
         table_header = {}
         duplicate_columns = []
         for line in header_info:
             if line.startswith("name"):
                 header_meta_info = {}
                 # get basic shortname and the full, non-differentiated info
-                shortname = longinfo = line_info = line.split("=")[1].strip()
+                shortname = longinfo = line_info = line.split("=", 1)[
+                    1
+                ].strip()
                 try:
-                    shortname, longinfo = line_info.split(":")
+                    shortname, longinfo = line_info.split(":", 1)
                 except IndexError:
                     pass
                 finally:
@@ -373,7 +432,7 @@ class Parameters(UserDict):
                     if shortname in list(table_header.keys()):
                         try:
                             duplicate_columns.append(
-                                int(line.split("=")[0].strip().split()[1])
+                                int(line.split("=", 1)[0].strip().split()[1])
                             )
                         except IndexError as error:
                             logger.error(
@@ -450,6 +509,9 @@ class Parameter:
         self.data = data
         self.metadata = metadata
         self.name = metadata["shortname"]
+        self.param = re.split(r"[,\s]", metadata["name"])[0]
+        self.sensor_number = 2 if metadata["name"][-1] == "2" else 1
+        self.unit = metadata["unit"]
         self.type = "data" if self.data.dtype in ["float", "int"] else "meta"
         self.parse_to_float()
         self.update_span()
